/**
 * Copyright Openmind http://www.openmindonline.it
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package it.openutils.magnoliastripes.tag;

import net.sourceforge.stripes.exception.StripesJspException;
import net.sourceforge.stripes.tag.FormTag;


/**
 * <p>
 * workaround needed to create a form tag with an "url" attribute different from the actionbean mapped uri. Could be
 * solved more cleanly by http://mc4j.org/jira/browse/STS-349, which will be available in stripes 1.5
 * </p>
 * @author fgiust
 * @version $Id: ExtendedFormTag.java 809 2008-05-05 23:11:37Z fgiust $
 */
public class ExtendedFormTag extends FormTag
{

    /**
     * Sets the 'action' attribute by inspecting the bean class provided and asking the current ActionResolver what the
     * appropriate URL is.
     * @param beanclass the Strin FQN of the class, or a Class representing the class
     * @throws StripesJspException if the URL cannot be determined for any reason, most likely because of a mis-spelled
     * class name, or a class that's not an ActionBean
     */
    @Override
    public void setBeanclass(Object beanclass) throws StripesJspException
    {
        String url = getActionBeanUrl(beanclass);
        if (url == null)
        {
            throw new StripesJspException("Could not determine action from 'beanclass' supplied. "
                + "The value supplied was '"
                + beanclass
                + "'. Please ensure that this bean type "
                + "exists and is in the classpath. If you are developing a page and the ActionBean "
                + "does not yet exist, consider using the 'action' attribute instead for now.");
        }
        else
        {
            String currentAction = get("action");

            setAction(url);

            if (currentAction != null)
            {
                set("action", currentAction);
            }
        }
    }
}
